﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TMcraft;
using System.Timers;
using System.Windows.Threading;
using System.Text.Json;
using System.IO;

namespace Positioner_Node_Setup
{
    /// <summary>
    /// Interaction logic for MainPage.xaml
    /// </summary>
    public partial class MainPage : UserControl,ITMcraftNodeEntry
    {
        TMcraftNodeAPI NodeUI = new TMcraftNodeAPI();
        private DispatcherTimer dispatcherTimer = new DispatcherTimer();
        private DispatcherTimer _autoCloseTimer = new DispatcherTimer();

        string _TMscript = string.Empty;
        bool fgSave = false;

        string strFileName = "Positioner_Cal";

        List<string> UserSaveData = new List<string>();
        Dictionary<string, object> NodeConfigs = new Dictionary<string, object>();

        Data? data = new Data();

        public class Data
        {
            public float[] AUXBase { get; set; }
            public float[] calibratedScore { get; set; }
            public float[] calibratedDH { get; set; }
            public int[] AUXJointLimit { get; set; }
            public int AUXSpeedLimit { get; set; }

            public Data()
            {
                AUXBase = new float[6];
                calibratedScore = new float[2];
                calibratedDH = new float[10];
                AUXJointLimit = new int[4];
                AUXSpeedLimit = 0;
            }
        }

        public MainPage()
        {
            InitializeComponent();
            InitializeTimer();
        }       

        public void InitializeNode(TMcraftNodeAPI _nodeUI)
        {
            NodeUI = _nodeUI; //connect TMflow
        }


        #region Warning Popup
        private void ShowWarningPopup(string strInput)
        {
            WarningText.Text = strInput;

            WarningPopup.IsOpen = true;

            _autoCloseTimer.Start();
        }

        private void AutoCloseTimer_Tick(object? sender, EventArgs e)
        {
            _autoCloseTimer.Stop();

            WarningPopup.IsOpen = false;
        }
        #endregion

        void InitializeTimer()
        {
            dispatcherTimer = new DispatcherTimer();
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(300);
            dispatcherTimer.Tick += TimerCheck;
            dispatcherTimer.Start();

            _autoCloseTimer = new DispatcherTimer
            {
                Interval = TimeSpan.FromSeconds(2)
            };
            _autoCloseTimer.Tick += AutoCloseTimer_Tick;
        }
        private void TimerCheck(object? sender, EventArgs e)
        {
            try
            {
                if (NodeUI == null || NodeUI.RobotStatusProvider == null)
                {
                    return;
                }

            }
            catch (Exception ex)
            {
                ShowWarningPopup(ex.Message);
            }

        }

        public void InscribeScript(ScriptWriteProvider scriptWriter)
        {
            if (fgSave) //Run this section before exit this node
            {
                for(int i = 0; i< UserSaveData.Count; i++) 
                {
                    scriptWriter.AppendScript(UserSaveData[i]);
                }
                
                string str = scriptWriter.GetScript();
                //MessageBox.Show(str);
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            try //Get the saved node configuration from its DataStorage and initialize the Node UI
            {

                if (NodeUI == null || NodeUI.RobotStatusProvider == null)
                {
                    ShowWarningPopup("[Load] No connection with TMflow...");
                    return;
                }
                else
                {
                    ReadDataFromFile();
                }
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }


        private void Btn_Save_Click(object sender, RoutedEventArgs e)
        {
            if(data != null)
            {
                if (data.calibratedScore[0] < 0.01)
                {
                    ShowWarningPopup("Do calibration first");
                    Btn_Save.IsEnabled = false;
                }
                else
                {
                    fgSave = true;
                    NodeUI.Close();
                }



            }

        }

        void ReadDataFromFile()
        {
            string[] strTemp = new string[1];
            string strContent = "";

            try
            {
                if (NodeUI == null || NodeUI.TextFileProvider == null)
                {
                    ShowWarningPopup("[Load] No connection with TMflow...");
                    return;
                }
                NodeUI.TextFileProvider.GetTextFileList(out strTemp);
                NodeUI.TextFileProvider.ReadTextFile(strFileName, out strContent);

                if (strTemp != null && strTemp.Length > 0)
                {
                    if (strTemp.Contains(strFileName))
                    {
                        //Calibration file detected
                        data = JsonSerializer.Deserialize<Data>(strContent);
                        if (data != null)
                        {
                            //Update table
                            Base_X.Text = data.AUXBase[0].ToString("0.00");
                            Base_Y.Text = data.AUXBase[1].ToString("0.00");
                            Base_Z.Text = data.AUXBase[2].ToString("0.00");
                            Base_RX.Text = data.AUXBase[3].ToString("0.00");
                            Base_RY.Text = data.AUXBase[4].ToString("0.00");
                            Base_RZ.Text = data.AUXBase[5].ToString("0.00");

                            AUX1_Score.Text = data.calibratedScore[0].ToString("0.0");
                            AUX2_Score.Text = data.calibratedScore[1].ToString("0.0");

                            AUX1_DH_Theta.Text = data.calibratedDH[0].ToString("0.00");
                            AUX1_DH_Alpha.Text = data.calibratedDH[1].ToString("0.00");
                            AUX1_DH_a.Text = data.calibratedDH[2].ToString("0.00");
                            AUX1_DH_d.Text = data.calibratedDH[3].ToString("0.00");

                            AUX2_DH_Theta.Text = data.calibratedDH[5].ToString("0.00");
                            AUX2_DH_Alpha.Text = data.calibratedDH[6].ToString("0.00");
                            AUX2_DH_a.Text = data.calibratedDH[7].ToString("0.00");
                            AUX2_DH_d.Text = data.calibratedDH[8].ToString("0.00");

                            

                            AUX1_JointLowerLimit.Text = data.AUXJointLimit[0].ToString("0");
                            AUX1_JointUpperLimit.Text = data.AUXJointLimit[1].ToString("0");
                            AUX2_JointLowerLimit.Text = data.AUXJointLimit[2].ToString("0");
                            AUX2_JointUpperLimit.Text = data.AUXJointLimit[3].ToString("0");

                            AUX_Speed.Text = data.AUXSpeedLimit.ToString("0");


                            //Save to Script
                            UserSaveData.Clear();
                            string _TMscript =
                                "Aux_ChangeBase({" +
                                data.AUXBase[0].ToString("0.000") + "," +
                                data.AUXBase[1].ToString("0.000") + "," +
                                data.AUXBase[2].ToString("0.000") + "," +
                                data.AUXBase[3].ToString("0.000") + "," +
                                data.AUXBase[4].ToString("0.000") + "," +
                                data.AUXBase[5].ToString("0.000") +
                                "})" +
                                "\n" +

                                "Aux_SetCalibratedDH({" +
                                data.calibratedDH[0].ToString("0.000") + "," +
                                data.calibratedDH[1].ToString("0.000") + "," +
                                data.calibratedDH[2].ToString("0.000") + "," +
                                data.calibratedDH[3].ToString("0.000") + "," +
                                "0" +
                                "," +
                                data.calibratedDH[5].ToString("0.000") + "," +
                                data.calibratedDH[6].ToString("0.000") + "," +
                                data.calibratedDH[7].ToString("0.000") + "," +
                                data.calibratedDH[8].ToString("0.000") + "," +
                                "0" +
                                "})" +
                                "\n" +

                                "Aux_SetJointOverLimit({" +
                                data.AUXJointLimit[0].ToString("0") + "," +
                                data.AUXJointLimit[1].ToString("0") + "," +
                                data.AUXJointLimit[2].ToString("0") + "," +
                                data.AUXJointLimit[3].ToString("0") +
                                "})" +
                                "\n" +

                                "Aux_SetJointSpeedLimit(" +
                                data.AUXSpeedLimit.ToString("0") +
                                ")" +
                                "\n" +

                                "Display(\"AUX calibrated\")" +
                                "";
                            UserSaveData.Add(_TMscript);

                            //Save to file for debugging
                            string currentDirectory = AppDomain.CurrentDomain.BaseDirectory;
                            string filePath = System.IO.Path.Combine(currentDirectory, "ScriptResult.txt");
                            File.WriteAllText(filePath, _TMscript);
                        }
                        else //No folder detected
                        {
                            ShowWarningPopup("Do calibration first");
                        }
                    }
                    else //No folder detected
                    {
                        ShowWarningPopup("Do calibration first");
                    }

                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("File reading error: " + ex.Message);
            }
        }
    }

}