﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using TMcraft;
using System.Diagnostics;
using System.Text.RegularExpressions;
using static System.Net.Mime.MediaTypeNames;
using System.Reflection;
using System.Text.Json;
using System.IO;
using System.Windows.Markup;
using System.Threading;
using static AUX_Toolbar.MainPage;

namespace AUX_Toolbar
{
    /// <summary>
    /// Interaction logic for MainPage.xaml
    /// </summary>
    public partial class MainPage : UserControl, ITMcraftToolbarEntry
    {
        TMcraftToolbarAPI ToolbarUI = new TMcraftToolbarAPI();

        int intSelectedJoint = 0; //0 means J7((AUX1); 1 means J8(AUX2)
        int intSelectedJointCalibration = 0; //0 means J7; 1 means J8
        int[] intAUXError = new int[] { 0, 0 }; //Error code
        int[] intAUXStatus = new int[] { 0, 0 }; //Status 


        float[] fCurrentRobotPosition_6x1 = { 0, 0, 0, 0, 0, 0 };
        float[] fCurrentRobotPosition_12x1 = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        float[] fCurrentRobotJointPosition_6x1 = { 0, 0, 0, 0, 0, 0 };
        float[] fCurrentRobotJointPosition_12x1 = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        float[] fTargetRobotPosition = { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f };
        float[] fCurrentAUXPosition = { 0.0f, 0.0f };
        float[] fCurrentAUXBase = { 0, 0, 0, 0, 0, 0 };

        private DispatcherTimer _timer = new DispatcherTimer();
        private DispatcherTimer _timer2 = new DispatcherTimer();
        private DispatcherTimer _autoCloseTimer = new DispatcherTimer();

        float fSpeed = 0.0f;
        bool bMoveFlag = false;
        bool bMoveFlag_Last = false;
        uint uCMDFeedback = 0;
        float[] fAUXPosition = { 0.0f, 0.0f };
        float[] fcalibratedScore = { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f };
        float[] fcalibratedDH = { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f };

        string strTemp = "";

        int[] iJointLimit = new int[4];

        int AUXSpeedLimit = 0;


        string strFileName = "Positioner_Cal";

        #region Warning Popup
        bool bIsBusy = false;

        private async void ShowWarningPopup(string strInput)
        {

            while (bIsBusy == true)
            {
                //wait loop
                await Task.Delay(100);
            }
            bIsBusy = true;

            // 設定警告訊息內容
            WarningText.Text = strInput;

            // 顯示 Popup
            WarningPopup.IsOpen = true;
            _autoCloseTimer.Start();



        }

        private void AutoCloseTimer_Tick(object? sender, EventArgs e)
        {

            _autoCloseTimer.Stop();

            WarningPopup.IsOpen = false;
            bIsBusy = false;
        }
        #endregion


        public class CalibrateArray
        {
            // Position: 6x1 float (can use a float array)
            public float[] Position { get; set; }

            // Position: 8x1 float (can use a float array)
            public float[] Joint { get; set; }

            // Position: 8x1 float (can use a float array)
            public float[] AUXJoint { get; set; }

            // Calibrated: boolean value
            public string Calibrated { get; set; }

            public CalibrateArray()
            {
                Position = new float[6] { 0, 0, 0, 0, 0, 0 }; // Initialize the Position array
                Joint = new float[6] { 0, 0, 0, 0, 0, 0 }; // Initialize the Joint array
                AUXJoint = new float[6] { 0, 0, 0, 0, 0, 0 }; // Initialize the AUXJoint array
                Calibrated = "UNCALIBRATED";      // Default value for Calibrated
            }
        }

        // Create an array of 20 Items
        CalibrateArray[] CalibratedArray = new CalibrateArray[20];


        public class Data
        {
            public float[] AUXBase { get; set; }
            public float[] calibratedScore { get; set; }
            public float[] calibratedDH { get; set; }
            public int[] AUXJointLimit { get; set; }
            public int AUXSpeedLimit { get; set; }

            public Data()
            {
                AUXBase = new float[6];
                calibratedScore = new float[] {-1,-1};
                calibratedDH = new float[10];
                AUXJointLimit = new int[] {-90, 90, -360, 360 };
                AUXSpeedLimit = 90;
            }
        }

        public MainPage()
        {
            InitializeArray();
            InitializeComponent();

            InitializeTimer();

        }

        private void InitializeArray()
        {
            // Initialize each Item
            for (int i = 0; i < CalibratedArray.Length; i++)
            {
                CalibratedArray[i] = new CalibrateArray();
            }
        }

        private void InitializeTimer()
        {
            _timer = new DispatcherTimer();
            _timer.Interval = TimeSpan.FromMilliseconds(200);
            _timer.Tick += Timer_Tick;
            _timer.Start(); 

            _timer2 = new DispatcherTimer();
            _timer2.Interval = TimeSpan.FromMilliseconds(1000);
            _timer2.Tick += Timer_Tick1000;
            _timer2.Start();


            _autoCloseTimer = new DispatcherTimer
            {
                Interval = TimeSpan.FromSeconds(2) // Hide after 2 sec
            };
            _autoCloseTimer.Tick += AutoCloseTimer_Tick;
        }
        private void Timer_Tick(object? sender, EventArgs e)
        {
            if ((ToolbarUI == null) || (ToolbarUI.AuxiliaryAxesProvider == null))
            {
                return;
            }

            try
            {
                ToolbarUI.AuxiliaryAxesProvider.GetAngle(out fCurrentAUXPosition);

                if (fCurrentAUXPosition == null)
                {
                    return;
                }
                LabelJ7.Content = fCurrentAUXPosition[0].ToString("0.0");
                LabelJ8.Content = fCurrentAUXPosition[1].ToString("0.0");

                ToolbarUI.RobotStatusProvider.GetCurrentPoseByRobotBase(out fCurrentRobotPosition_6x1);
                ToolbarUI.RobotStatusProvider.GetCurrentPoseByJointAngle(out fCurrentRobotJointPosition_6x1);

                Array.Copy(fCurrentRobotPosition_6x1, fCurrentRobotPosition_12x1, 6);
                Array.Copy(fCurrentRobotJointPosition_6x1, fCurrentRobotJointPosition_12x1, 6);

                fCurrentRobotPosition_12x1[6] = fCurrentAUXPosition[0];
                fCurrentRobotPosition_12x1[7] = fCurrentAUXPosition[1];

                fCurrentRobotJointPosition_12x1[6] = fCurrentAUXPosition[0];
                fCurrentRobotJointPosition_12x1[7] = fCurrentAUXPosition[1];

                labelCurrentPosition.Text =
                    "[ " + fCurrentRobotPosition_12x1[0].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[1].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[2].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[3].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[4].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[5].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[6].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[7].ToString("0.0") + " ]";

                if ((bMoveFlag == true) && (bMoveFlag_Last == false)) //High edge of moving command
                {
                    ToolbarUI.RobotJogProvider.HoldPlayKeyToRun(true); //Start moving
                }
                else if ((bMoveFlag == false) && (bMoveFlag_Last == true)) //Low edge of moving command
                {
                    ToolbarUI.RobotJogProvider.HoldPlayKeyToRun(false); //End moving
                    ToolbarUI.RobotJogProvider.StopJog();
                }

                if (bMoveFlag == true)
                {
                    ToolbarUI.RobotJogProvider.KeepJogging();

                }
                
                bMoveFlag_Last = bMoveFlag;



            }
            catch (Exception ex)
            {
                textBlockDebug.Text = ex.Message;
            }

        }

        private void Timer_Tick1000(object? sender, EventArgs e)
        {
            if (TabSETTING.IsSelected == true)
            {
                // Activate when it is in specfic tab
                UpdateAUXStatus();
            }


        }

        void ReadTable()
        {
            iJointLimit[0] = int.Parse(textBox_J7_Lowerlimit.Text);
            iJointLimit[1] = int.Parse(textBox_J7_Upperlimit.Text);
            iJointLimit[2] = int.Parse(textBox_J8_Lowerlimit.Text);
            iJointLimit[3] = int.Parse(textBox_J8_Upperlimit.Text);

            //AUX Speed
            AUXSpeedLimit = int.Parse(textBox_AUXSpeedlimit.Text);
        }

        void WriteToRobot()
        {
            uint uCMDFeedback = 0;


            try
            {

                if ((ToolbarUI == null) || (ToolbarUI.AuxiliaryAxesProvider == null))
                {
                    return;
                }

                //textBox_J7_Lowerlimit
                uCMDFeedback = ToolbarUI.AuxiliaryAxesProvider.SetJointOverLimit(iJointLimit);
                if (uCMDFeedback == 0)
                {
                    //ShowWarningPopup("The joint limits are set");
                }
                else
                {
                    ShowWarningPopup("Joint limits are invalid or no AUX attached");
                }


                //AUX Speed
                uCMDFeedback = ToolbarUI.AuxiliaryAxesProvider.SetJointSpeedLimit(AUXSpeedLimit);
                if (uCMDFeedback == 0)
                {
                    //ShowWarningPopup("The speed limit is set");
                }
                else
                {
                    ShowWarningPopup("Speed limit is invalid or no AUX attached");
                }

            }
            catch (Exception ex)
            {
                textBlockDebug_Status.Text = ex.Message;
            }
        }

        public void UpdateAUXStatus()
        {
            string strErrorCode = "";
            string strStatus = "";
            string[] strTempArray = new string[2];

            if ((ToolbarUI == null) || (ToolbarUI.AuxiliaryAxesProvider == null))
            {
                return;
            }
            try
            {
                ToolbarUI.AuxiliaryAxesProvider.GetErrorStatus(out intAUXError);
                ToolbarUI.AuxiliaryAxesProvider.GetOperationStatus(out intAUXStatus);

                #region Get information
                strTemp = "";
                ToolbarUI.AuxiliaryAxesProvider.GetVendorID(out strTempArray);
                for (int i = 0; i < strTempArray.Length; i++)
                {
                    strTemp += strTempArray[i] + ", ";
                }
                labelAUXVenderID.Text = strTemp;
                strTemp = "";
                strTempArray = new string[2];


                ToolbarUI.AuxiliaryAxesProvider.GetDeviceSN(out strTempArray);
                for (int i = 0; i < strTempArray.Length; i++)
                {
                    strTemp += strTempArray[i] + ", ";
                }
                labelAUXDeviceSN.Text = strTemp;
                strTemp = "";
                strTempArray = new string[2];

                ToolbarUI.AuxiliaryAxesProvider.GetManufactureDeviceName(out strTempArray);
                for (int i = 0; i < strTempArray.Length; i++)
                {
                    strTemp += strTempArray[i] + ", ";
                }
                labelAUXDeviceName.Text = strTemp;
                strTemp = "";
                strTempArray = new string[2];

                ToolbarUI.AuxiliaryAxesProvider.GetManufactureSWVersion(out strTempArray);
                for (int i = 0; i < strTempArray.Length; i++)
                {
                    strTemp += strTempArray[i] + ", ";
                }
                labelAUXSoftwareVersion.Text = strTemp;
                #endregion

                strErrorCode = "";
                strStatus = "";

                if ((intAUXStatus != null) && (intAUXStatus.Length > 0))
                {
                    if (((intAUXStatus[0] & 0b00001000) != 0) || ((intAUXStatus[0] & 0b01000000) != 0))
                    {
                        strStatus = "Error or Warning";
                    }
                    else
                    {
                        strStatus = "Running";
                    }

                    if (intAUXError[0] == 0)
                    {
                        strErrorCode = "No Error";
                    }
                    else
                    {
                        strErrorCode = intAUXError[0].ToString("X");
                    }

                    if (intAUXStatus.Length > 1) // Multiple axes
                    {
                        strStatus += ", ";
                        strErrorCode += ", ";

                        if (((intAUXStatus[1] & 0b00001000) != 0) || ((intAUXStatus[1] & 0b01000000) != 0))
                        {
                            strStatus += "Error or Warning";
                        }
                        else
                        {
                            strStatus += "Running";
                        }

                        if (intAUXError[1] == 0)
                        {
                            strErrorCode += "No Error";
                        }
                        else
                        {
                            strErrorCode += intAUXError[1].ToString("X");
                        }
                    }

                    
                    labelAUXStatus.Text = strStatus;
                    labelAUXError.Text = strErrorCode;
                }



                bool[] bTemp = new bool[1];
                ToolbarUI.AuxiliaryAxesProvider.CheckJointOverLimit(out bTemp);
                if(bTemp.Length == 1) //If only AUX1 installed
                {
                    if(bTemp[0]==true)
                    {
                        textBox_J7_Lowerlimit.Background = new SolidColorBrush(Colors.Red);
                        textBox_J7_Upperlimit.Background = new SolidColorBrush(Colors.Red);
                    }
                    else
                    {
                        textBox_J7_Lowerlimit.Background = new SolidColorBrush(Colors.White);
                        textBox_J7_Upperlimit.Background = new SolidColorBrush(Colors.White);
                    }
                }
                else if(bTemp.Length == 2) //If AUX1 + AUX2 installed
                {
                    if (bTemp[0] == true)
                    {
                        textBox_J7_Lowerlimit.Background = new SolidColorBrush(Colors.Red);
                        textBox_J7_Upperlimit.Background = new SolidColorBrush(Colors.Red);
                    }
                    else
                    {
                        textBox_J7_Lowerlimit.Background = new SolidColorBrush(Colors.White);
                        textBox_J7_Upperlimit.Background = new SolidColorBrush(Colors.White);
                    }

                    if (bTemp[1] == true)
                    {
                        textBox_J8_Lowerlimit.Background = new SolidColorBrush(Colors.Red);
                        textBox_J8_Upperlimit.Background = new SolidColorBrush(Colors.Red);
                    }
                    else
                    {
                        textBox_J8_Lowerlimit.Background = new SolidColorBrush(Colors.White);
                        textBox_J8_Upperlimit.Background = new SolidColorBrush(Colors.White);
                    }
                }

                textBlockDebug_Status.Text = "";
            }
            catch (Exception ex)
            {
                textBlockDebug.Text = ex.Message;
            }
        }

        public void InitializeToolbar(TMcraftToolbarAPI _toolbarUI)
        {
            ToolbarUI = _toolbarUI;
        }

        private void UserControl_Loaded(object sender, EventArgs e)
        { 

            try
            {
                if (ToolbarUI == null || ToolbarUI.RobotStatusProvider == null)
                {
                    ShowWarningPopup("No Connection with TMflow");
                }
                else
                {

                }

                //Context Initialization
                UpdateComboBoxCalibrationPoint();
                ReadfromFile();
                ReadTable();
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            
        }





        private void RadioButtonJ7_Checked(object sender, RoutedEventArgs e)
        {
            intSelectedJoint = 0;
        }
        private void RadioButtonJ8_Checked(object sender, RoutedEventArgs e)
        {
            intSelectedJoint = 1;
        }

        private void comboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            switch (comboBoxSpeed.SelectedIndex)
            {
                case 0:
                    fSpeed = 15;
                    break;

                case 1:
                    fSpeed = 1;
                    break;

                default:
                    break;
            }
            
        }
        #region Holding Event

        void JogFWD()
        {
            if ((ToolbarUI == null) || (ToolbarUI.AuxiliaryAxesProvider == null))
            {
                return;
            }

            float fJointTarget = 0;

            switch(intSelectedJoint)
            {
                case 0:
                    fJointTarget = (float)iJointLimit[1]; //AUX1 upper limit
                    break;
                case 1:
                    fJointTarget = (float)iJointLimit[3]; //AUX2 upper limit
                    break;
            }

            uCMDFeedback = ToolbarUI.AuxiliaryAxesProvider.JogSingleAxis(intSelectedJoint, fJointTarget, (int)fSpeed, false);
            bMoveFlag = true;
        }
        void JogBWD()
        {
            if ((ToolbarUI == null) || (ToolbarUI.AuxiliaryAxesProvider == null))
            {
                return;
            }

            float fJointTarget = 0;

            switch (intSelectedJoint)
            {
                case 0:
                    fJointTarget = (float)iJointLimit[0]; //AUX1 lower limit
                    break;
                case 1:
                    fJointTarget = (float)iJointLimit[2]; //AUX2 lower limit
                    break;
            }

            uCMDFeedback = ToolbarUI.AuxiliaryAxesProvider.JogSingleAxis(intSelectedJoint, fJointTarget, (int)fSpeed, false);
            bMoveFlag = true;
        }

        void JogEnd()
        {
            bMoveFlag = false;
        }

        void JogToTarget()
        {

            if ((ToolbarUI == null) || (ToolbarUI.AuxiliaryAxesProvider == null))
            {
                return;
            }

            if (TabOPERATING.IsSelected) 
            {
                float.TryParse(TextBoxJ7.Text, out fAUXPosition[0]);
                float.TryParse(TextBoxJ8.Text, out fAUXPosition[1]);
            }
            else if(TabCALIBRATION.IsSelected)
            {
                
            }
            uCMDFeedback = ToolbarUI.AuxiliaryAxesProvider.JogAllAxes(fAUXPosition, (int)fSpeed, false);

            bMoveFlag = true;
        }

        private void imageMinus_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            JogBWD();
        }

        private void imagePlus_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            JogFWD();
        }

        private void imagePlay_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            JogToTarget();
        }

        private void imageMinus_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            JogEnd();
        }

        private void imagePlus_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            JogEnd();
        }

        private void imagePlay_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            JogEnd();
        }


        private void imageMinus_PreviewTouchUp(object sender, TouchEventArgs e)
        {
            JogEnd();
        }

        private void imagePlus_PreviewTouchUp(object sender, TouchEventArgs e)
        {
            JogEnd();
        }

        private void imagePlus_PreviewTouchDown(object sender, TouchEventArgs e)
        {
            JogFWD();
        }

        private void imageMinus_PreviewTouchDown(object sender, TouchEventArgs e)
        {
            JogBWD();
        }

        private void imagePlay_PreviewTouchUp(object sender, TouchEventArgs e)
        {
            JogEnd();
        }

        private void imagePlay_PreviewTouchDown(object sender, TouchEventArgs e)
        {
            JogToTarget();
        }

        private void imagePlayCalibrate_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            JogToTarget();
        }

        private void imagePlayCalibrate_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            JogEnd();
        }

        private void imagePlayCalibrate_PreviewTouchUp(object sender, TouchEventArgs e)
        {
            JogEnd();
        }

        private void imagePlayCalibrate_PreviewTouchDown(object sender, TouchEventArgs e)
        {
            JogToTarget();
        }
        #endregion

        private void TextBoxJ7_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            // Int and float only
            e.Handled = !IsValidInput(e.Text);
        }

        private void TextBoxJ7_PreviewKeyDown(object sender, KeyEventArgs e)
        {
        }

        private void TextBoxJ8_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            e.Handled = !IsValidInput(e.Text);
        }

        // Int and float only
        private bool IsValidInput(string text)
        {
            return Regex.IsMatch(text, @"^-?[0-9]*\.?[0-9]*$");
        }

        private void TextBoxJ8_PreviewKeyDown(object sender, KeyEventArgs e)
        {
        }



        private async void TextBoxJ7_TextChanged(object sender, TextChangedEventArgs e)
        {
            await Task.Delay(1000);
            if (float.TryParse(TextBoxJ7.Text, out float value))
            {
                if (value > (float)iJointLimit[1]) //Upper limit for AUX1
                {

                    TextBoxJ7.Text = iJointLimit[1].ToString();
                }
                else if (value < (float)iJointLimit[0])
                {
                    TextBoxJ7.Text = iJointLimit[0].ToString();
                }
            }
            else
            {
                TextBoxJ7.Text = "0.0";
            }
        }

        private async void TextBoxJ8_TextChanged(object sender, TextChangedEventArgs e)
        {
            await Task.Delay(1000);
            if (float.TryParse(TextBoxJ8.Text, out float value))
            {
                if (value > (float)iJointLimit[3]) //Upper limit for AUX2
                {

                    TextBoxJ8.Text = iJointLimit[3].ToString();
                }
                else if (value < (float)iJointLimit[2])
                {
                    TextBoxJ8.Text = iJointLimit[2].ToString();
                }
            }
            else
            {
                TextBoxJ8.Text = "0.0";
            }
        }

        private void comboBoxJointSelection_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

            UpdateComboBoxCalibrationPoint();
            UpdateCalibration();

        }
        void UpdateComboBoxCalibrationPoint()
        {
            if (comboBoxCalibrationPoint == null)
            {
                return;
            }
            comboBoxCalibrationPoint.Items.Clear();

            switch (comboBoxJointSelection.SelectedIndex)
            {
                case 0:
                    intSelectedJointCalibration = 0;
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P1 (0)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P2 (+30)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P3 (+60)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P4 (-30)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P5 (-60)" });
                    comboBoxCalibrationPoint.SelectedIndex = 0;
                    break;

                case 1:
                    intSelectedJointCalibration = 1;
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P1 (0)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P2 (+72)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P3 (+144)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P4 (+216)" });
                    comboBoxCalibrationPoint.Items.Add(new ComboBoxItem { Content = "P5 (+288)" });
                    comboBoxCalibrationPoint.SelectedIndex = 0;
                    break;
            }
        }

        void UpdateCalibration()
        {
            int iIndex = 0;
            if (intSelectedJointCalibration == 0)
            {
                iIndex = 0;
            }
            else if (intSelectedJointCalibration == 1)
            {
                iIndex = 10;
            }

            labelCalibratedStatusP1.Content = CalibratedArray[0 + iIndex].Calibrated;
            labelCalibratedStatusP2.Content = CalibratedArray[1 + iIndex].Calibrated;
            labelCalibratedStatusP3.Content = CalibratedArray[2 + iIndex].Calibrated;
            labelCalibratedStatusP4.Content = CalibratedArray[3 + iIndex].Calibrated;
            labelCalibratedStatusP5.Content = CalibratedArray[4 + iIndex].Calibrated;

        }

        private void buttonRecord_Click(object sender, RoutedEventArgs e)
        {
            int iIndex = 0;
            string strTemp = "";

            if(intSelectedJointCalibration==0)
            {
                iIndex = 0;
            }
            else if(intSelectedJointCalibration==1)
            {
                iIndex = 10;
            }

            if(RadioButtonP1.IsChecked==true)
            {
                Array.Copy(fCurrentRobotPosition_6x1, CalibratedArray[0 + iIndex].Position, 6);
                Array.Copy(fCurrentAUXPosition, CalibratedArray[0 + iIndex].AUXJoint, 2);
                CalibratedArray[0+iIndex].Calibrated = "CALIBRATED";
            }
            if (RadioButtonP2.IsChecked == true)
            {
                Array.Copy(fCurrentRobotPosition_6x1, CalibratedArray[1 + iIndex].Position, 6);
                Array.Copy(fCurrentAUXPosition, CalibratedArray[1 + iIndex].AUXJoint, 2);
                CalibratedArray[1 + iIndex].Calibrated = "CALIBRATED";
            }
            if (RadioButtonP3.IsChecked == true)
            {
                Array.Copy(fCurrentRobotPosition_6x1, CalibratedArray[2 + iIndex].Position, 6);
                Array.Copy(fCurrentAUXPosition, CalibratedArray[2 + iIndex].AUXJoint, 2);
                CalibratedArray[2 + iIndex].Calibrated = "CALIBRATED";
            }
            if (RadioButtonP4.IsChecked == true)
            {
                Array.Copy(fCurrentRobotPosition_6x1, CalibratedArray[3 + iIndex].Position, 6);
                Array.Copy(fCurrentAUXPosition, CalibratedArray[3 + iIndex].AUXJoint, 2);
                CalibratedArray[3 + iIndex].Calibrated = "CALIBRATED";
            }
            if (RadioButtonP5.IsChecked == true)
            {
                Array.Copy(fCurrentRobotPosition_6x1, CalibratedArray[4 + iIndex].Position, 6);
                Array.Copy(fCurrentAUXPosition, CalibratedArray[4 + iIndex].AUXJoint, 2);
                CalibratedArray[4 + iIndex].Calibrated = "CALIBRATED";
            }
            strTemp = "[ " + fCurrentRobotPosition_12x1[0].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[1].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[2].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[3].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[4].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[5].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[6].ToString("0.0") + ", "
                    + fCurrentRobotPosition_12x1[7].ToString("0.0") + " ]";

            ShowWarningPopup("Calibrated: "+ strTemp);
            UpdateCalibration();
        }



        private void comboBoxCalibrationPoint_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(comboBoxJointSelection.SelectedIndex == 0)
            {
                switch (comboBoxCalibrationPoint.SelectedIndex)
                {
                    case 0:
                        RadioButtonP1.IsChecked = true;
                        fAUXPosition[0] = 0;
                        break;
                    case 1:
                        RadioButtonP2.IsChecked = true;
                        fAUXPosition[0] = 30;
                        break;
                    case 2:
                        RadioButtonP3.IsChecked = true;
                        fAUXPosition[0] = 60;
                        break;
                    case 3:
                        RadioButtonP4.IsChecked = true;
                        fAUXPosition[0] = -30;
                        break;
                    case 4:
                        RadioButtonP5.IsChecked = true;
                        fAUXPosition[0] = -60;
                        break;
                }
            }
            else if(comboBoxJointSelection.SelectedIndex == 1)
            {
                switch (comboBoxCalibrationPoint.SelectedIndex)
                {
                    case 0:
                        RadioButtonP1.IsChecked = true;
                        fAUXPosition[1] = 0;
                        break;
                    case 1:
                        RadioButtonP2.IsChecked = true;
                        fAUXPosition[1] = 72;
                        break;
                    case 2:
                        RadioButtonP3.IsChecked = true;
                        fAUXPosition[1] = 144;
                        break;
                    case 3:
                        RadioButtonP4.IsChecked = true;
                        fAUXPosition[1] = 216;
                        break;
                    case 4:
                        RadioButtonP5.IsChecked = true;
                        fAUXPosition[1] = 288;
                        break;
                }
            }


        }
        void StartCalibrtion()
        {
            int iIndex = 0;
            uint uReceivedData = 0;

            if (ToolbarUI == null || ToolbarUI.RobotStatusProvider == null)
            {
                ShowWarningPopup("No Connection with TMflow");
                return;
            }
            uReceivedData = ToolbarUI.AuxiliaryAxesProvider.ResetCalibration();

            if (uReceivedData != 0)
            {
                ShowWarningPopup("Error while reseting calibration");
                return;
            }

            iIndex = 0;
            float[] fZeroArray = new float[] { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f };

            //AUX1
            for (int i = 0; i < 5; i++)
            {
                if (CalibratedArray[i + iIndex].Calibrated == "CALIBRATED")
                {
                    uReceivedData = ToolbarUI.AuxiliaryAxesProvider.AddCalibrationPos(0, CalibratedArray[i + iIndex].Position, fZeroArray, CalibratedArray[i + iIndex].AUXJoint.Take(2).ToArray());

                    if (uReceivedData != 0)
                    {
                        MessageBox.Show("Error while adding calibration [" + (i).ToString() + "] :" +
                        "Position:[" + string.Join(", ", CalibratedArray[i + iIndex].Position) + "], [" +
                        "AUX:[" + string.Join(", ", CalibratedArray[i + iIndex].AUXJoint.Take(2).ToArray()) + "]"
                        );
                        return;
                    }
                }
            }

            //AUX2
            iIndex = 10; //Swtich to from AUX1 to AUX2


            for (int i = 0; i < 5; i++)
            {
                if (CalibratedArray[i + iIndex].Calibrated == "CALIBRATED")
                {
                    uReceivedData = ToolbarUI.AuxiliaryAxesProvider.AddCalibrationPos(1, CalibratedArray[i + iIndex].Position, fZeroArray, CalibratedArray[i + iIndex].AUXJoint.Take(2).ToArray());

                    if (uReceivedData != 0)
                    {
                        MessageBox.Show("Error while adding calibration [" + (i).ToString() + "] :" +
                        "Position:[" + string.Join(", ", CalibratedArray[i + iIndex].Position) + "], [" +
                        "AUX:[" + string.Join(", ", CalibratedArray[i + iIndex].AUXJoint.Take(2).ToArray()) + "]"
                        );
                        return;
                    }
                }
            }

            //Start calibration
            uReceivedData = ToolbarUI.AuxiliaryAxesProvider.StartCalibration(out fCurrentAUXBase, out fcalibratedScore, out fcalibratedDH);
            if (uReceivedData != 0)
            {
                MessageBox.Show("Error while getting calculation result");
                return;
            }


            uReceivedData = ToolbarUI.AuxiliaryAxesProvider.SetCurrentAuxBase(fCurrentAUXBase);
            if (uReceivedData != 0)
            {
                MessageBox.Show("Error while writting result");
                return;
            }

            uReceivedData = ToolbarUI.AuxiliaryAxesProvider.SetDH(fcalibratedDH);
            if (uReceivedData != 0)
            {
                MessageBox.Show("Error while writting DH");
                return;
            }

            strTemp = "Done, result = " + fCurrentAUXBase[0].ToString("0.0") + ", " +
            fCurrentAUXBase[1].ToString("0.0") + ", " +
            fCurrentAUXBase[2].ToString("0.0") + ", " +
            fCurrentAUXBase[3].ToString("0.0") + ", " +
            fCurrentAUXBase[4].ToString("0.0") + ", " +
            fCurrentAUXBase[5].ToString("0.0") + ", " +
            "Score = {" + fcalibratedScore[0].ToString("0.0") + ", " + fcalibratedScore[1].ToString("0.0") + "}" +
            "DH = {";

            for (int i = 0; i < 10; i++)
            {
                if (i < 9)
                {
                    strTemp += fcalibratedDH[i].ToString("0.0") + ", ";
                }
                else
                {
                    strTemp += fcalibratedDH[i].ToString("0.0") + "}";
                }
            }

            ShowWarningPopup(strTemp);
        }
        private void buttonCalibrate_Click(object sender, RoutedEventArgs e)
        {
            StartCalibrtion();
            WriteToRobot();
            WriteToFile_Calibration();
        }

        void WriteToFile_Setting()
        {
            string strContent = "";
            string[] strTemp = new string[1];
            var dict = new Dictionary<string, object>();
            Data? data = new Data();

            try
            {
                if (ToolbarUI == null || ToolbarUI.TextFileProvider == null)
                {
                    ShowWarningPopup("[Load] No connection with TMflow...");
                    return;
                }
                ToolbarUI.TextFileProvider.GetTextFileList(out strTemp);
                ToolbarUI.TextFileProvider.ReadTextFile(strFileName, out strContent);

                if (strTemp != null && strTemp.Length > 0)  //Check if the calibration file is existed
                {
                    if (strTemp.Contains(strFileName))
                    {
                        //Calibration file detected
                        data = JsonSerializer.Deserialize<Data>(strContent);
                        if (data != null)
                        {
                            dict = JsonSerializer.Deserialize<Dictionary<string, object>>(strContent) ?? new Dictionary<string, object>();
                        }
                    }
                }
                

                if (data != null)
                {
                    data.AUXJointLimit = iJointLimit;
                    data.AUXSpeedLimit = AUXSpeedLimit;

                    dict["AUXJointLimit"] = data.AUXJointLimit;
                    dict["AUXSpeedLimit"] = data.AUXSpeedLimit;
                }

                var options = new JsonSerializerOptions { WriteIndented = true };
                string updatedJson = JsonSerializer.Serialize(dict, options);
                ToolbarUI.TextFileProvider.WriteTextFile(strFileName, updatedJson);

                ShowWarningPopup("Setting Saved Successfully");
            }
            catch (JsonException ex)
            {
                textBlockDebug_Status.Text = "JSON file error: " + ex.Message;
                data = new Data();
            }
            catch (Exception ex)
            {
                textBlockDebug_Status.Text = "Failed to save setting: " + ex.Message;
            }
        }

        void WriteToFile_Calibration()
        {
            string strContent = "";
            string[] strTemp = new string[1];
            var dict = new Dictionary<string, object>();
            Data? data = new Data();

            try
            {

                if (ToolbarUI == null || ToolbarUI.TextFileProvider == null)
                {
                    ShowWarningPopup("[Load] No connection with TMflow...");
                    return;
                }
                ToolbarUI.TextFileProvider.GetTextFileList(out strTemp);
                ToolbarUI.TextFileProvider.ReadTextFile(strFileName, out strContent);

                
                if (strTemp != null && strTemp.Length > 0)  //Check if the calibration file is existed
                {
                    if (strTemp.Contains(strFileName))
                    {
                        //Calibration file detected
                        data = JsonSerializer.Deserialize<Data>(strContent);
                        if (data != null)
                        {
                            dict = JsonSerializer.Deserialize<Dictionary<string, object>>(strContent) ?? new Dictionary<string, object>();
                        }
                    }
                }

                if (data != null)
                {
                    data.AUXBase = fCurrentAUXBase;
                    data.calibratedScore = fcalibratedScore;
                    data.calibratedDH = fcalibratedDH;

                    dict["AUXBase"] = data.AUXBase;
                    dict["calibratedScore"] = data.calibratedScore;
                    dict["calibratedDH"] = data.calibratedDH;
                }
                
                var options = new JsonSerializerOptions { WriteIndented = true };
                string updatedJson = JsonSerializer.Serialize(dict, options);
                ToolbarUI.TextFileProvider.WriteTextFile(strFileName, updatedJson);

                ShowWarningPopup("Calibration Saved Successfully");

            }
            catch (JsonException ex)
            {
                textBlockDebug_Status.Text = "JSON file error: " + ex.Message;
                data = new Data();
            }
            catch (Exception ex)
            {
                textBlockDebug_Status.Text = "Failed to save calibration: " + ex.Message;
            }
        }


        void ReadfromFile()
        {
            string strContent = "";
            string[] strTemp = new string[1];
            var dict = new Dictionary<string, object>();
            Data? data = new Data();
            var options = new JsonSerializerOptions
            {
                WriteIndented = true
            };

            try
            {
                string jsonString = JsonSerializer.Serialize(data, options);

                if (ToolbarUI == null || ToolbarUI.TextFileProvider == null)
                {
                    ShowWarningPopup("[Load] No connection with TMflow...");
                    return;
                }
                ToolbarUI.TextFileProvider.GetTextFileList(out strTemp);
                ToolbarUI.TextFileProvider.ReadTextFile(strFileName, out strContent);

                if(strTemp.Length == 0)
                {
                    ToolbarUI.TextFileProvider.NewTextFile(strFileName, jsonString);
                }

                if (strTemp != null && strTemp.Length > 0)  //Check if the calibration file is existed
                {
                    if (strTemp.Contains(strFileName))
                    {
                        //Calibration file detected
                        data = JsonSerializer.Deserialize<Data>(strContent);
                    }
                    else
                    {
                        ToolbarUI.TextFileProvider.NewTextFile(strFileName, jsonString);
                    }

                    if (data != null)
                    {
                        textBox_J7_Lowerlimit.Text = data.AUXJointLimit[0].ToString();
                        textBox_J7_Upperlimit.Text = data.AUXJointLimit[1].ToString();
                        textBox_J8_Lowerlimit.Text = data.AUXJointLimit[2].ToString();
                        textBox_J8_Upperlimit.Text = data.AUXJointLimit[3].ToString();

                        textBox_AUXSpeedlimit.Text = data.AUXSpeedLimit.ToString();
                    }
                }

            }
            catch (JsonException ex)
            {
                textBlockDebug_Status.Text = $"JSON file error: {ex.Message}";
                data = new Data();
            }
            catch (Exception ex)
            {
                textBlockDebug_Status.Text = $"JSON file error: {ex.Message}";
            }
        }

        private void Button_Save1(object sender, RoutedEventArgs e)
        {
            ReadTable();
            WriteToRobot();
            WriteToFile_Setting();
        }
    }
}