﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using TMcraft;
using System.Timers;
using System.Windows.Threading;
using System.IO;
using System.Diagnostics;

namespace Positioner_Node
{
    /// <summary>
    /// Interaction logic for MainPage.xaml
    /// </summary>
    public partial class MainPage : UserControl, ITMcraftNodeEntry
    {
        TMcraftNodeAPI NodeUI = new TMcraftNodeAPI();
        private DispatcherTimer dispatcherTimer = new DispatcherTimer();
        private DispatcherTimer _autoCloseTimer = new DispatcherTimer();

        string _TMscript = string.Empty;
        bool fgSave = false;

        float[] fCurrentRobotPosition_6x1 = new float[6] { 0, 0, 0, 0, 0, 0 };
        float[] fCurrentAUXPosition_2x1 = new float[2] { 0, 0 };
        float[] fCurrentRobotPosition_12x1 = new float[12] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        float[] fCurrentRobotPosition_P1_12x1 = new float[12] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        float[] fCurrentRobotPosition_P2_12x1 = new float[12] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        int iPointMode = 0;
        int iSpeed = 100;
        int iSpeedMode = 0;
        string strSpeedVariable = "";

        List<string> UserSaveData = new List<string>();
        Dictionary<string, object> NodeConfigs = new Dictionary<string, object>();

        string strSpeed = "";
        string strSpeedMode = "";

        public MainPage()
        {
            InitializeComponent();
            InitializeTimer();


        }

        public void InitializeNode(TMcraftNodeAPI _nodeUI)
        {
            NodeUI = _nodeUI; //connect TMflow
        }


        #region Warning Popup
        private void ShowWarningPopup(string strInput)
        {
            WarningText.Text = strInput;

            WarningPopup.IsOpen = true;

            _autoCloseTimer.Start();
        }

        private void AutoCloseTimer_Tick(object? sender, EventArgs e)
        {
            _autoCloseTimer.Stop();

            WarningPopup.IsOpen = false;
        }
        #endregion

        void InitializeTimer()
        {
            dispatcherTimer = new DispatcherTimer();
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(300);
            dispatcherTimer.Tick += TimerCheck;
            dispatcherTimer.Start();


            _autoCloseTimer = new DispatcherTimer
            {
                Interval = TimeSpan.FromSeconds(2)
            };
            _autoCloseTimer.Tick += AutoCloseTimer_Tick;
        }
        private void TimerCheck(object? sender, EventArgs e)
        {
            try
            {
                if (NodeUI == null || NodeUI.RobotStatusProvider == null)
                {
                    return;
                }
                NodeUI.AuxiliaryAxesProvider.GetCurrentPoseByCurrentBase(out fCurrentRobotPosition_6x1);
                NodeUI.AuxiliaryAxesProvider.GetAngle(out fCurrentAUXPosition_2x1);
                

                if (NodeUI.AuxiliaryAxesProvider == null)
                {
                    //No AUX attached, in TMflow simulator for example.
                    //NodeUI.RobotStatusProvider.GetCurrentPoseByCurrentBase(out fCurrentRobotPosition_6x1);
                }
                else
                {
                    //Attached to a real AUX
                    
                }

                if (fCurrentAUXPosition_2x1 != null)
                {
                    fCurrentRobotPosition_12x1[6] = fCurrentAUXPosition_2x1[0]; //Read J7
                    fCurrentRobotPosition_12x1[7] = fCurrentAUXPosition_2x1[1]; //Read J8
                }
                else
                {
                    fCurrentRobotPosition_12x1[6] = 999; //Read J7
                    fCurrentRobotPosition_12x1[7] = 999; //Read J8
                }

                Array.Copy(fCurrentRobotPosition_6x1, fCurrentRobotPosition_12x1, 6);

                labelCurrentPosition.Content =
                "[" + fCurrentRobotPosition_12x1[0].ToString("0.0") + ", "
                + fCurrentRobotPosition_12x1[1].ToString("0.0") + ", "
                + fCurrentRobotPosition_12x1[2].ToString("0.0") + ", "
                + fCurrentRobotPosition_12x1[3].ToString("0.0") + ", "
                + fCurrentRobotPosition_12x1[4].ToString("0.0") + ", "
                + fCurrentRobotPosition_12x1[5].ToString("0.0") + "], ["
                + fCurrentRobotPosition_12x1[6].ToString("0.0") + ", "
                + fCurrentRobotPosition_12x1[7].ToString("0.0") + ", "
                + "]";


                /*
                #region
                if ((bMoveFlag == true) && (bMoveFlag_Last == false)) //High edge of moving command
                {
                    NodeUI.RobotJogProvider.
                        //.HoldPlayKeyToRun(true); //Start moving
                }
                else if ((bMoveFlag == false) && (bMoveFlag_Last == true)) //Low edge of moving command
                {
                    NodeUI.RobotJogProvider.HoldPlayKeyToRun(false); //End moving
                    NodeUI.RobotJogProvider.StopJog();
                }

                if (bMoveFlag == true)
                {
                    NodeUI.RobotJogProvider.KeepJogging();

                }

                bMoveFlag_Last = bMoveFlag;
                #endregion
                */
            }
            catch (Exception ex)
            {
                ShowWarningPopup(ex.Message);
            }

        }

        public void InscribeScript(ScriptWriteProvider scriptWriter)
        {
            if (fgSave) //Run this section before exit this node
            {
                for (int i = 0; i < UserSaveData.Count; i++)
                {
                    scriptWriter.AppendScript(UserSaveData[i]);
                }

                string str = scriptWriter.GetScript();
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            try //Get the saved node configuration from its DataStorage and initialize the Node UI
            {
                Border_Point2.Visibility = Visibility.Hidden;
                if (NodeUI == null || NodeUI.RobotStatusProvider == null)
                {
                    ShowWarningPopup("[Load] No connection with TMflow...");
                    return;
                }
                else
                {
                    uint result = 0;
                    string TMErrMsg = string.Empty;
                    result = NodeUI.DataStorageProvider.GetAllData(out NodeConfigs);


                    if (result == 0)
                    {
                        if (NodeConfigs.Count == 0)//Node is implemented at the first time.
                        {

                        }
                        else if (NodeConfigs.Count >= 1)
                        {
                            string str_Content = string.Empty;
                            result += NodeUI.DataStorageProvider.GetData("Position_P1_12x1", out str_Content);
                            ConvertStringToFloat(str_Content, out fCurrentRobotPosition_P1_12x1);
                            UpdatePosition_P1(fCurrentRobotPosition_P1_12x1);

                            result += NodeUI.DataStorageProvider.GetData("Position_P2_12x1", out str_Content);
                            ConvertStringToFloat(str_Content, out fCurrentRobotPosition_P2_12x1);
                            UpdatePosition_P2(fCurrentRobotPosition_P2_12x1);

                            result += NodeUI.DataStorageProvider.GetData("MoveMode", out str_Content);
                            if (str_Content == "0") //Line mode
                            {
                                comboBoxModeSelect.SelectedIndex = 0;
                            }
                            else if (str_Content == "1") //Circle mode
                            {
                                comboBoxModeSelect.SelectedIndex = 1;
                            }
                            else if (str_Content == "2") //PTP mode
                            {
                                comboBoxModeSelect.SelectedIndex = 2;
                            }
                            result += NodeUI.DataStorageProvider.GetData("Speed", out str_Content);
                            textBox_Speed.Text = str_Content;

                            result += NodeUI.DataStorageProvider.GetData("SpeedMode", out str_Content);
                            if (str_Content == "0") //ABS
                            {
                                comboBoxSpeedSelect.SelectedIndex = 0;
                            }
                            else if (str_Content == "1") //Project %
                            {
                                comboBoxSpeedSelect.SelectedIndex = 1;
                            }
                            if (result != 0)
                            {
                                NodeUI.GetErrMsg(result, out TMErrMsg);
                                MessageBox.Show("GetData Failure: " + TMErrMsg);
                            }
                            else
                            {

                            }
                        }
                    }
                    else
                    {
                        string errMsg = string.Empty;
                        TMcraftErr TMe = NodeUI.GetErrMsg(result, out errMsg);

                        if (TMe != TMcraftErr.OK)
                        {
                            MessageBox.Show(result.ToString() + " ; TMcraftErr : " + TMe.ToString());
                        }
                        else
                        {
                            MessageBox.Show(result.ToString() + " : " + errMsg);
                        }
                    }
                }


            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void ConvertStringToFloat(string strInput, out float[] fInput)
        {
            strInput = strInput.Trim('{', '}');
            string[] strValues = strInput.Split(',');
            fInput = strValues.Select(s => float.Parse(s)).ToArray();
        }

        private void Btn_Save_Click(object sender, RoutedEventArgs e)
        {
            string _TMscript = "";
            UserSaveData.Clear();

            string strPosition_P1_12x1 = string.Join(",", fCurrentRobotPosition_P1_12x1.Select(x => x.ToString("0.000")));
            string strPosition_P2_12x1 = string.Join(",", fCurrentRobotPosition_P2_12x1.Select(x => x.ToString("0.000")));



            CheckTextBoxValue(textBox_Speed);

            if (iSpeed > 0)
            {
                strSpeed = iSpeed.ToString();
            }
            else if(strSpeedVariable != "")
            {
                strSpeed = strSpeedVariable;
            }

            if (iPointMode == 0) //Line mode
            { 
                _TMscript = "Aux_Line(\"" 
                + strSpeedMode
                + "\",{"
                + fCurrentRobotPosition_P1_12x1[0].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[1].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[2].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[3].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[4].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[5].ToString("0.00") + "}"
                + ","
                + strSpeed
                + ",500,100,true,{"
                + fCurrentRobotPosition_P1_12x1[6].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[7].ToString("0.00") + "}"
                + ")";

                NodeConfigs["MoveMode"] = 0;
            }
            else if (iPointMode == 1)
            {
                _TMscript = "Aux_Circle(\""
                + strSpeedMode
                + "\",{"
                + fCurrentRobotPosition_P1_12x1[0].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[1].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[2].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[3].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[4].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[5].ToString("0.00") + "},{"

                + fCurrentRobotPosition_P2_12x1[0].ToString("0.00") + ","
                + fCurrentRobotPosition_P2_12x1[1].ToString("0.00") + ","
                + fCurrentRobotPosition_P2_12x1[2].ToString("0.00") + ","
                + fCurrentRobotPosition_P2_12x1[3].ToString("0.00") + ","
                + fCurrentRobotPosition_P2_12x1[4].ToString("0.00") + ","
                + fCurrentRobotPosition_P2_12x1[5].ToString("0.00") + "},"
                + strSpeed
                + ",500,100,0,true,{"
                + fCurrentRobotPosition_P2_12x1[6].ToString("0.0") + ","
                + fCurrentRobotPosition_P2_12x1[7].ToString("0.0") + "}"
                + ")";

                NodeConfigs["MoveMode"] = 1;
            }
            else if (iPointMode == 2)
            {
                _TMscript = "Aux_PTP(\"CPP\",{"
                + fCurrentRobotPosition_P1_12x1[0].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[1].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[2].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[3].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[4].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[5].ToString("0.00") + "}"
                + ","
                + strSpeed
                + ",500,100,true,{"
                + fCurrentRobotPosition_P1_12x1[6].ToString("0.00") + ","
                + fCurrentRobotPosition_P1_12x1[7].ToString("0.00") + "}"
                + ")";

                NodeConfigs["MoveMode"] = 2;
            }
            NodeConfigs["SpeedMode"] = iSpeedMode.ToString();

            UserSaveData.Add(_TMscript);

            

            if (NodeUI != null)
            {
                uint result = 0;
                result += NodeUI.DataStorageProvider.SaveData("Position_P1_12x1", strPosition_P1_12x1);
                result += NodeUI.DataStorageProvider.SaveData("Position_P2_12x1", strPosition_P2_12x1);

                result += NodeUI.DataStorageProvider.SaveData("Speed", strSpeed);

                result += NodeUI.DataStorageProvider.SaveData("MoveMode", NodeConfigs["MoveMode"].ToString());

                result += NodeUI.DataStorageProvider.SaveData("SpeedMode", NodeConfigs["SpeedMode"].ToString());

                string TMErrMsg = string.Empty;


                if (result == 0) //If no error ( == 0 )
                {
                    fgSave = true;
                    NodeUI.Close();
                }
                else
                {
                    string errMsg = string.Empty;
                    TMcraftErr TMe = NodeUI.GetErrMsg(result, out errMsg);

                    if (TMe != TMcraftErr.OK)
                    {
                        MessageBox.Show(result.ToString() + " ; TMcraftErr : " + TMe.ToString());
                    }
                    else
                    {
                        MessageBox.Show(result.ToString() + " : " + errMsg);
                    }
                }


            }
            else
            {
                ShowWarningPopup("[Save] No connection with TMflow...");
            }

            //Save to file for debugging
            string currentDirectory = AppDomain.CurrentDomain.BaseDirectory;
            string filePath = System.IO.Path.Combine(currentDirectory, "ScriptResult.txt");
            File.WriteAllText(filePath, _TMscript);
        }

        void UpdatePosition_P1(float[] fInput)
        {

            labelRecordedPosition_P1.Content =
                    "[" + fInput[0].ToString("0.00") + ", "
                    + fInput[1].ToString("0.00") + ", "
                    + fInput[2].ToString("0.00") + ", "
                    + fInput[3].ToString("0.00") + ", "
                    + fInput[4].ToString("0.00") + ", "
                    + fInput[5].ToString("0.00") + "], ["
                    + fInput[6].ToString("0.00") + ", "
                    + fInput[7].ToString("0.00")
                    + "]";
        }

        void UpdatePosition_P2(float[] fInput)
        {
            labelRecordedPosition_P2.Content =
                    "[" + fInput[0].ToString("0.00") + ", "
                    + fInput[1].ToString("0.00") + ", "
                    + fInput[2].ToString("0.00") + ", "
                    + fInput[3].ToString("0.00") + ", "
                    + fInput[4].ToString("0.00") + ", "
                    + fInput[5].ToString("0.00") + "], ["
                    + fInput[6].ToString("0.00") + ", "
                    + fInput[7].ToString("0.00")
                    + "]";
        }

        private void button_Click_P1(object sender, RoutedEventArgs e)
        {
            Array.Copy(fCurrentRobotPosition_12x1, fCurrentRobotPosition_P1_12x1, fCurrentRobotPosition_12x1.Length);
            UpdatePosition_P1(fCurrentRobotPosition_12x1);
            ShowWarningPopup("POINT 1 RECORDED");
        }

        private void button_Click_P2(object sender, RoutedEventArgs e)
        {
            Array.Copy(fCurrentRobotPosition_12x1, fCurrentRobotPosition_P2_12x1, fCurrentRobotPosition_12x1.Length);
            UpdatePosition_P2(fCurrentRobotPosition_12x1);
            ShowWarningPopup("POINT 2 RECORDED");

        }

        private void comboBoxCalibrationPoint_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                if (Border_Point2 != null)
                {
                    switch (comboBoxModeSelect.SelectedIndex)
                    {
                        case 0: //Line mode
                            iPointMode = 0;
                            Border_Point2.Visibility = Visibility.Hidden;
                            fCurrentRobotPosition_P2_12x1 = new float[] { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, }; //Clear the data for P2
                            UpdatePosition_P2(fCurrentRobotPosition_P2_12x1);
                            break;

                        case 1: //Circle mode
                            iPointMode = 1;
                            Border_Point2.Visibility = Visibility.Visible;
                            break;

                        case 2: //PTP mode
                            iPointMode = 2;
                            Border_Point2.Visibility = Visibility.Hidden;
                            fCurrentRobotPosition_P2_12x1 = new float[] { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, }; //Clear the data for P2
                            UpdatePosition_P2(fCurrentRobotPosition_P2_12x1);
                            break;
                    }
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }



        }

        private void Button_Click_P1_Move(object sender, RoutedEventArgs e)
        {

        }
        private void Button_Click_P2_Move(object sender, RoutedEventArgs e)
        {

        }

        private void comboBoxSpeedSelect_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            CheckTextBoxValue(textBox_Speed);

            if(comboBoxSpeedSelect.SelectedIndex==0) //Speed = ABS
            {
                iSpeedMode = 0;
                strSpeedMode = "CDP";
            }
            else //Speed = Project %
            {
                iSpeedMode = 1;
                strSpeedMode = "CPP";
            }
        }

        private void textBox_Speed_SelectionChanged(object sender, RoutedEventArgs e)
        {
            TextBox? textBox = sender as TextBox;
            if (textBox != null)
            {
                if (string.IsNullOrEmpty(textBox.Text))
                {
                    textBox.Text = "10"; //Default value
                }
            }

        }
        private void CheckTextBoxValue(TextBox textBox_Input)
        {
            string input = "";
            if(textBox_Input != null)
            {
                input = textBox_Input.Text;
            }
            
            if (string.IsNullOrEmpty(input))
            {
                iSpeed = 0;
                strSpeedVariable = "";
            }
            else if (int.TryParse(input, out int number))
            {
                if (number < 0)
                {
                    number = 0;
                }

                iSpeed = number;
                strSpeedVariable = "";
            }
            else
            {
                iSpeed = 0;
                strSpeedVariable = input;
            }
        }

        private void ToggleButton_P1_Move_Checked(object sender, RoutedEventArgs e)
        {
            TextBlock_P1_Move.Text = "Hold PLAY to Move";
            TextBlock_P2_Move.Text = "Move";
            JogToTarget(1);
        }

        private void ToggleButton_P1_Move_Unchecked(object sender, RoutedEventArgs e)
        {
            TextBlock_P1_Move.Text = "Move";
            TextBlock_P2_Move.Text = "Move";
            JogEnd();
        }

        private void ToggleButton_P2_Move_Checked(object sender, RoutedEventArgs e)
        {
            TextBlock_P1_Move.Text = "Move";
            TextBlock_P2_Move.Text = "Hold PLAY to Move";
            JogToTarget(2);
        }

        private void ToggleButton_P2_Move_Unchecked(object sender, RoutedEventArgs e)
        {
            TextBlock_P1_Move.Text = "Move";
            TextBlock_P2_Move.Text = "Move";
            JogEnd();
        }
        void JogToTarget(int iPoint)
        {
            if (NodeUI == null || NodeUI.AuxiliaryAxesProvider == null)
            {
                return;
            }
            switch(iPoint)
            {
                case 1:
                    NodeUI.AuxiliaryAxesProvider.JogByBase(fCurrentRobotPosition_P1_12x1.Take(6).ToArray(), 5, fCurrentRobotPosition_P1_12x1.Skip(6).Take(2).ToArray());
                    break;
                case 2:
                    NodeUI.AuxiliaryAxesProvider.JogByBase(fCurrentRobotPosition_P2_12x1.Take(6).ToArray(), 5, fCurrentRobotPosition_P2_12x1.Skip(6).Take(2).ToArray());
                    break;
                default:
                    break;
            }

            
        }
        void JogEnd()
        {
            if (NodeUI == null || NodeUI.AuxiliaryAxesProvider == null)
            {
                return;
            }

            NodeUI.RobotJogProvider.StopJog();

        }


    }


}